package main

import (
	"io"
	"log"
	"net/http"
	"sync"
)

var (
	// variables to set during build-time
	debugging = ""
	version   = "0.0-undefined"
	buildtime = "0000-00-00T00:00:00+0000"

	// actual business end of the device
	paths = &sync.Map{}
)

// Transfer holds a single tranferable connection to be read
type Transfer struct {
	reader        io.ReadCloser
	done          chan struct{}
	contentlength string
}

// NewTransfer returns a ready to use Transfer and it's writer
func NewTransfer(contentlength string) (transfer Transfer, writer io.WriteCloser) {
	var reader io.ReadCloser
	reader, writer = io.Pipe()

	transfer = Transfer{
		reader:        reader.(io.ReadCloser),
		contentlength: contentlength,
		done:          make(chan struct{}),
	}

	return
}

// Queue is where posts and gets can exchange transfers
type Queue struct {
	ch    chan Transfer
	posts int32
	gets  int32
}

func pathHandler(w http.ResponseWriter, r *http.Request) {
	pathID := r.URL.Path

	if len(pathID) < 2 {
		w.WriteHeader(400)
		w.Write([]byte("path to short"))
		return
	}

	if r.Method == "GET" {
		log.Printf("%s [GET] Connected", pathID)
		handleGet(pathID, w, r)
	} else {
		log.Printf("%s [POST] Connected", pathID)
		if h := r.Header.Get("X-Pathway"); h == "pubsub" {
			debug("%s [PUBSUB] Upgrade POST to PUBSUB", pathID)
			handlePubSub(pathID, r)
		} else {
			handlePost(pathID, r)
		}
	}
}

func okHandler(w http.ResponseWriter, r *http.Request) {
	w.WriteHeader(http.StatusOK)
	w.Write([]byte("ok"))
}

func emptyHandler(w http.ResponseWriter, r *http.Request) {
	w.WriteHeader(http.StatusOK)
}

func main() {
	log.Printf("pathway version:%s buildtime:%s", version, buildtime)

	http.HandleFunc("/health", okHandler)
	http.HandleFunc("/favicon.ico", emptyHandler)
	http.HandleFunc("/robots.txt", emptyHandler)
	http.HandleFunc("/", pathHandler)

	err := http.ListenAndServe(":8080", nil)
	if err != nil {
		log.Println(err)
	}
}

func debug(msg string, args ...interface{}) {
	if len(debugging) > 0 {
		log.Printf(msg, args...)
	}
}
